@@ -108,38 +108,40 @@ pub fn draw_tree (
             false => {},
         };
         // Display name
-        let mut element = Element::new("text");
-        element.assign("x", index.x-5.0);
-        element.assign("y", index.y+10.0);
-        element.assign("class", "gene");
-        let txt  = Text::new(&index.name);
-        element.append(txt);
-        element.assign("transform","rotate(90 ".to_owned()+&(index.x - 5.0).to_string()
-        + ","+&(index.y + 10.0).to_string()+")");
-        match tree.is_leaf(index.idx) {
-            true => g.append(element),
-            _   =>  match options.gene_internal {
-                        true =>  g.append(element),
-                        false => {},
-                    },
-        };
+        if index.visible {
+            let mut element = Element::new("text");
+            element.assign("x", index.x-5.0);
+            element.assign("y", index.y+10.0);
+            element.assign("class", "gene");
+            let txt  = Text::new(&index.name);
+            element.append(txt);
+            element.assign("transform","rotate(90 ".to_owned()+&(index.x - 5.0).to_string()
+            + ","+&(index.y + 10.0).to_string()+")");
+            match tree.is_leaf(index.idx) {
+                true => g.append(element),
+                _   =>  match options.gene_internal {
+                            true =>  g.append(element),
+                            false => {},
+                        },
+            };
 
-        // Display support
-        let mut element = Element::new("text");
-        element.assign("x", index.x-15.0);
-        element.assign("y", index.y+10.0);
-        element.assign("class", "support");
-        let txt  = Text::new(&index.support);
-        element.append(txt);
-        element.assign("transform","rotate(90 ".to_owned()+&(index.x - 15.0).to_string()
-        + ","+&(index.y + 10.0).to_string()+")");
-        match tree.is_leaf(index.idx) {
-            true => {},
-            _   =>  match options.support {
-                        true =>  g.append(element),
-                        false => {},
-                    },
-        };
+            // Display support
+            let mut element = Element::new("text");
+            element.assign("x", index.x-15.0);
+            element.assign("y", index.y+10.0);
+            element.assign("class", "support");
+            let txt  = Text::new(&index.support);
+            element.append(txt);
+            element.assign("transform","rotate(90 ".to_owned()+&(index.x - 15.0).to_string()
+            + ","+&(index.y + 10.0).to_string()+")");
+            match tree.is_leaf(index.idx) {
+                true => {},
+                _   =>  match options.support {
+                            true =>  g.append(element),
+                            false => {},
+                        },
+            };
+        }
     }
     let mut transfo: String = "translate(  ".to_owned();
     transfo.push_str(&( x_viewbox).to_string());
@@ -224,7 +226,9 @@ pub fn draw_sptree_gntrees (
                                                  index.width/2.0, max_gene_y - index.y,
                                                  config.species_color.to_string(),
                                                  config.species_opacity.to_string());
-                    g.append(chemin);
+                    if sp_tree.arena[p].visible {
+                        g.append(chemin)
+                    };
                 }
                 match  index.e {
                     Event::Loss => {
@@ -287,7 +291,9 @@ pub fn draw_sptree_gntrees (
                 element.append(txt);
                 element.assign("transform","rotate(90 ".to_owned() + &index.x.to_string()
                 + "," + &index.y.to_string() + ")" );
-                g.append(element);
+                if index.visible {
+                    g.append(element);
+                }
             },
             false => {
                 match options.species_internal {
@@ -299,7 +305,9 @@ pub fn draw_sptree_gntrees (
                         element.append(txt);
                         element.assign("transform","rotate(90 ".to_owned()+&index.x.to_string()
                         + "," + &index.y.to_string() + ")" );
-                        g.append(element);
+                        if index.visible {
+                            g.append(element);
+                        }
                     },
                     false => {},
                 };
@@ -353,13 +361,13 @@ pub fn draw_sptree_gntrees (
                                         gene_color.to_string(),
                                         transfer_opacity,
                                         config.bezier.to_string().parse::<f32>().unwrap(),
-                                        true),
+                                        1),
                                 Event::BifurcationOut => get_chemin_transfer(index.x,index.y,
                                         n.x,n.y,
                                         gene_color.to_string(),
                                         transfer_opacity,
                                         config.bezier.to_string().parse::<f32>().unwrap(),
-                                        true),
+                                        2),
                                 _ => panic!("Wrong recPhyloXML feature.
                                 The father node should be BranchingOut or
                                 BifurcationOut, but I found a {:?}\n{:?}",n.e,n),
@@ -491,7 +499,7 @@ pub fn draw_sptree_gntrees (
               "red".to_string(),
               opacity,
               config.bezier.to_string().parse::<f32>().unwrap(),
-              false
+              0
           );
           g.append(chemin);
 
@@ -634,7 +642,7 @@ pub fn get_chemin_carre (x1: f32, y1:f32,x2: f32, y2:f32, c:String, o:String, st
 
 // Draw a transfer path between x1,y1 ad x2,y2
 pub fn get_chemin_transfer (x1: f32, y1:f32,x2: f32, y2:f32, c:String, o:String,b:f32,
-    stroke:bool) -> Path {
+    stroke:i32) -> Path {
     // Arrivee du point: un peu avant pour dessiner la fleche
     let initial_y1 = y1 ;
     let y1 = y1 - PIPEBLOCK;
@@ -666,8 +674,10 @@ pub fn get_chemin_transfer (x1: f32, y1:f32,x2: f32, y2:f32, c:String, o:String,
     .set("opacity", o)
     .set("stroke-width", GTHICKNESS);
     let path = match stroke {
-        true => path.set("stroke-dasharray","1, 1"),
-        false => path,
+        2 => path.set("stroke-dasharray","4, 1"),
+        1 => path.set("stroke-dasharray","1, 1"),
+        0 => path,
+        _ => path.set("stroke-dasharray","1, 1"),
     };
     let path  = path.set("d", data);
     path
